/*
  Macro           Test_MarkStream_with_copy_commands
  Author          Carlo Hogeveen
  Website         eCarlo.nl/tse
  Compatibility   Windows TSE v4 upwards,
                  except some old, irrelevant beta versions of TSE.
  Version         v1   22 Apr 2024


  This test macro demonstartes how MarkStream() works with
    Copy()/Paste()
    CopyBlock()
    CopyToWinClip()()/PasteFromWinClip()

  The RemoveTrailingWhite setting is temporarily set to OFF,
  so that end-of-line spaces are never automatically removed.

  What I noticed:
  - If the macro is run in TSE 4.49e and TSE 4.50rc21, then it does show the
    CopyToWinClip()/PasteFromWinClip() change that Knud encountered.
  - That change can arguably called an improvement, because CopyToWinClip() now
    copies as many characters as are visibly marked.
    But the change did not unify CopyToWinClip()'s behavior with that of TSE's
    other copy commands when used after MarkStream().
  - All three copy commands (still) show inconsistent behavior when used after
    MarkStream() and past the end of the line.

  An impact analysis revealed no macro dependency on MarkStream in Semware's or
  my macros.
  In a user's interactive usage there is of course a dependency,
  and as Knud encountered there can be a dependency in another user's macro.

  This macro's purpose is just to show how MarkStream() and TSE's copy
  commands work together.

  It is neither a request for change nor an objection to change.

*/


integer proc line1_length()
  integer len = 0
  PushLocation()
  GotoLine(1)
  len = CurrLineLen()
  PopLocation()
  return(len)
end line1_length


proc Main()
  integer i                       = 0
  integer old_RemoveTrailingWhite = Set(RemoveTrailingWhite, OFF)
  integer old_SpecialEffects      = Query(SpecialEffects)
  integer old_UnMarkAfterPaste    = Set(UnMarkAfterPaste, OFF)
  integer tmp1_id                 = 0
  integer tmp2_id                 = 0

  #if EDITOR_VERSION >= 4400h
    Set(SpecialEffects, Query(SpecialEffects) | _CENTER_POPUPS_)
  #endif
  #ifndef INTERNAL_VERSION        // TSE v4.41.44, 18 Jan 2021.
    #define INTERNAL_VERSION 0
  #endif
  #if INTERNAL_VERSION >= 12369   // TSE v4.50rc7, 29 Jul 2023.
    Set(SpecialEffects, Query(SpecialEffects) & (MAXINT ^ _TOP_RT_POPUPS_))
  #endif

  PushPosition()
  tmp1_id = NewFile()
  InsertText('hello world')
  tmp2_id = NewFile()

  for i = 1 to 13
    UnMarkBlock()
    GotoBufferId(tmp1_id)
    BegLine()
    MarkStream()
    GotoColumn(i)
    MarkStream()
    UpdateDisplay(_ALL_WINDOWS_REFRESH_)
    Warn(i; 'characters marked with MarkStream() ...')
    Copy()
    GotoBufferId(tmp2_id)
    EmptyBuffer()
    Paste()
    UpdateDisplay(_ALL_WINDOWS_REFRESH_)
    Warn('Copy()/Paste() of'; i; 'characters to'; line1_length(); 'line1 characters.')
  endfor

  for i = 1 to 13
    UnMarkBlock()
    GotoBufferId(tmp1_id)
    BegLine()
    MarkStream()
    GotoColumn(i)
    MarkStream()
    UpdateDisplay(_ALL_WINDOWS_REFRESH_)
    Warn(i; 'characters marked with MarkStream() ...')
    GotoBufferId(tmp2_id)
    EmptyBuffer()
    CopyBlock()
    UpdateDisplay(_ALL_WINDOWS_REFRESH_)
    Warn('CopyBlock() of'; i; 'characters to'; line1_length(); 'line1 characters.')
  endfor

  for i = 1 to 13
    UnMarkBlock()
    GotoBufferId(tmp1_id)
    BegLine()
    MarkStream()
    GotoColumn(i)
    MarkStream()
    UpdateDisplay(_ALL_WINDOWS_REFRESH_)
    Warn(i; 'characters marked with MarkStream() ...')
    CopyToWinClip()
    GotoBufferId(tmp2_id)
    EmptyBuffer()
    PasteFromWinClip()
    UpdateDisplay(_ALL_WINDOWS_REFRESH_)
    Warn('CopyToWinClip()/PasteFromWinClip() of'; i; 'characters to'; line1_length(); 'line1 characters.')
  endfor

  PopPosition()
  AbandonFile(tmp1_id)
  AbandonFile(tmp2_id)
  Set(RemoveTrailingWhite, old_RemoveTrailingWhite)
  Set(SpecialEffects     , old_SpecialEffects)
  Set(UnMarkAfterPaste   , old_UnMarkAfterPaste)
  PurgeMacro(CurrMacroFilename())
end Main

